/**
 * The following file will hold sets of dislay graph rendering functions.
 */
#include <tulip/IntegerProperty.h>
#include <tulip/ColorProperty.h>
#include <tulip/GraphProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/ForEach.h>
#include "OpenCloseHierarchy.h"
#include "ColourShapeDefaults.h"
#include "DisplayGraphRendering.h"

using namespace std;
using namespace tlp;

//================================================================
DisplayGraphRendering::
DisplayGraphRendering (tlp::Graph *displayGraph, tlp::Graph *hierarchy,
		       tlp::Graph *inputGraph) :
  displayGraph (displayGraph),
  hierarchy (hierarchy),
  leafColour (LEAF_COLOUR),
  edgeColour (EDGE_COLOUR)
{
}

//================================================================
void DisplayGraphRendering::ensureNodeColours () {
  ColorProperty *colours = 
    displayGraph->getRoot()->getProperty<ColorProperty>("viewColor");
  IntegerProperty *displayToHier = 
    displayGraph->getProperty<IntegerProperty> (displayToHierProperty);
  node curNode;
  forEach (curNode, displayGraph->getNodes()) {
    node hierNode (displayToHier->getNodeValue (curNode));
    Color nodeColor = colours->getNodeValue (hierNode);
    nodeColor.setA (NODE_ALPHA);
    colours->setNodeValue (curNode, nodeColor);
  }//end forEach
}//end ensureNodeColours

//================================================================
void DisplayGraphRendering::ensureNodeRenderOrder () {
  GraphProperty *hierarchyMetaGraphs =
    hierarchy->getProperty<GraphProperty> ("viewMetaGraph");
  IntegerProperty *displayToHier = 
    displayGraph->getProperty<IntegerProperty> (displayToHierProperty);
  DoubleProperty *viewMetric =
    displayGraph->getProperty<DoubleProperty> ("viewMetric");
  node curNode;
  forEach (curNode, displayGraph->getNodes()) {
    node hierNode (displayToHier->getNodeValue (curNode));
    Graph *hierarchySubGraph = hierarchyMetaGraphs->getNodeValue (hierNode);
    //negatives so large nodes rendered first
    if (hierarchySubGraph != NULL) {
      viewMetric->setNodeValue (curNode,
				-1*hierarchySubGraph->numberOfNodes());
    }
    else
      viewMetric->setNodeValue (curNode, -1); //a leaf is one node
  }//end forEach
}//end ensureNodeRenderOrder

//================================================================
void DisplayGraphRendering::ensureEdgeColours () {
  ColorProperty *colours = 
    displayGraph->getProperty<ColorProperty>("viewColor");
  edge curEdge;
  forEach (curEdge, displayGraph->getEdges()) {
    colours->setEdgeValue (curEdge, edgeColour);
  }
}

//================================================================
#define MIN_FRACTION (1.0/10.0)
void DisplayGraphRendering::ensureEdgeSizes () {
  SizeProperty *sizes = 
    displayGraph->getProperty<SizeProperty> ("viewSize");
  edge curEdge;
  forEach (curEdge, displayGraph->getEdges()) {
    node source = displayGraph->source (curEdge);
    node target = displayGraph->target (curEdge);
    Size sourceSize = sizes->getNodeValue (source);
    Size targetSize = sizes->getNodeValue (target);
    double minW = (sourceSize.getW() < targetSize.getW()) ?
      sourceSize.getW() : targetSize.getW();
    double minH = (sourceSize.getH() < targetSize.getH()) ?
      sourceSize.getH() : targetSize.getH();
    sizes->setEdgeValue (curEdge, Size (MIN_FRACTION*minW, 
					MIN_FRACTION*minH, 0.5));
  }//end forEach
}//end ensureEdgeSizes

