/**
 * Functions used to modify display graph rendering.
 */

#ifndef __DISPLAY_GRAPH_RENDERING_H__
#define __DISPLAY_GRAPH_RENDERING_H__

#include <tulip/Color.h>
#include <tulip/Graph.h>
#include <tulip/MetaGraphHierarchyWidget.h>

class DisplayGraphRendering {
 private:
  tlp::Graph *displayGraph;
  tlp::Graph *hierarchy;
  tlp::Color leafColour;
  tlp::Color edgeColour;

 public:
  DisplayGraphRendering (tlp::Graph *displayGraph, tlp::Graph *hierarchy,
			 tlp::Graph *inputGraph);

  inline ~DisplayGraphRendering () {}
  
  /**
   * The following function ensures node in the display graph are
   * coloured the same as they are in the hierarchy.
   */
  void ensureNodeColours ();

  /**
   * The following function sets the edge colours of the graph.
   */
  void ensureEdgeColours ();

  /**
   * The following function sets the render order to render the labels
   * of large metanodes first.
   */
  void ensureNodeRenderOrder ();

  /**
   * The following function sets edge sizes to a fraction of the
   * minimum node adjacency size.
   */
  void ensureEdgeSizes ();

  inline void setLeafColour (const tlp::Color &newLeafColour) {
    leafColour = newLeafColour;
  }

  inline void setEdgeColour (const tlp::Color &newEdgeColour) {
    edgeColour = newEdgeColour;
  }
};
#endif
