#include <tulip/ForEach.h>
#include <tulip/LayoutProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/IntegerProperty.h>
#include "OpenCloseHierarchy.h"
#include "EdgeBundling.h"

using namespace std;
using namespace tlp;
 
//=================================================================
EdgeBundling::EdgeBundling (Graph *displayGraph,
			    OpenCloseHierarchy *openCloseHierarchy) :
  displayGraph (displayGraph),
  openCloseHierarchy (openCloseHierarchy)
{}

//=================================================================
void EdgeBundling::computeBend (const edge &curEdge) {
  LayoutProperty *displayLayout = 
    displayGraph->getProperty<LayoutProperty> ("viewLayout");
  node displaySource = displayGraph->source (curEdge);
  node displayTarget = displayGraph->target (curEdge);
  node hierarchySource = openCloseHierarchy->getHierarchyNode (displaySource);
  node hierarchyTarget = openCloseHierarchy->getHierarchyNode (displayTarget);
  vector<node> sourcePath, targetPath;
  openCloseHierarchy->getPath (hierarchySource, sourcePath);
  openCloseHierarchy->getPath (hierarchyTarget, targetPath);

  //find LCA
  unsigned int pathMeta = 0;
  while (sourcePath[pathMeta] == targetPath[pathMeta])
    ++pathMeta;
 
  vector<Coord> controlPoint;
  Graph *lcaGraph = 
    openCloseHierarchy->getContainingHierarchyGraph (sourcePath[pathMeta]);
  SizeProperty *hierarchySizes = 
    lcaGraph->getProperty<SizeProperty> ("viewSize");
  Size sourceSize = hierarchySizes->getNodeValue (sourcePath[pathMeta]);
  Size targetSize = hierarchySizes->getNodeValue (targetPath[pathMeta]);
  sourceSize.setD (0);
  targetSize.setD (0);
  Coord sourceGlobal = openCloseHierarchy->getGlobalPos (sourcePath[pathMeta]);
  Coord targetGlobal = openCloseHierarchy->getGlobalPos (targetPath[pathMeta]);
  Coord edgeVectorNorm = (targetGlobal - sourceGlobal);
  edgeVectorNorm /= edgeVectorNorm.norm ();
  Coord sourceBoundaryPoint = sourceGlobal + 
    edgeVectorNorm*(sourceSize.norm ()/2.0);
  Coord targetBoundaryPoint= targetGlobal - 
    edgeVectorNorm*(targetSize.norm ()/2.0);
  Coord midpoint = (sourceBoundaryPoint + targetBoundaryPoint)/2.0;
  controlPoint.push_back (midpoint);
  displayLayout->setEdgeValue (curEdge, controlPoint);
}//end computeBend

//=================================================================
void EdgeBundling::computeBends () {
  edge curEdge;
  forEach (curEdge, displayGraph->getEdges ())
    computeBend (curEdge);
}//end computeBends

//=================================================================
void EdgeBundling::toggleEdgeBundling (bool on) {
  IntegerProperty *edgeShape = 
    displayGraph->getProperty<IntegerProperty> ("viewShape");
  if (on) {
    edgeShape->setAllEdgeValue (SPLINESHAPE);
    computeBends ();
  }//end if
  else {
    edgeShape->setAllEdgeValue (POLYLINESHAPE);
    LayoutProperty *viewLayout =
      displayGraph->getProperty<LayoutProperty> ("viewLayout");
    viewLayout->setAllEdgeValue (vector<Coord> ());
    openCloseHierarchy->resynchDisplayGraph ();
  }//end else
}//end toggleBundling
