/**
 * The following class implements functions to preform a form of
 * edge bundling on the GCA edge.
 */

#ifndef __EDGE_BUNDLING_H__
#define __EDGE_BUNDLING_H__

#include <vector>
#include <tulip/Graph.h>

class OpenCloseHierarchy;

class EdgeBundling {
 public:
  /**
   * Constructor.
   * \param displayGraph The graph where edge bundling is displayed.
   * \param openCloseHierarchy The openClose hierarchy object for 
   * dispaly graph and nodes.
   */
  EdgeBundling (tlp::Graph *displayGraph, 
		OpenCloseHierarchy *openCloseHierarchy);

  /**
   * Turns edge bundling on or off.  It will also compute all bends
   * for the scene if turned on and reset bends if turned off.
   * \param on If true turns bundling on.
   */
  void toggleEdgeBundling (bool on);

 private:
  
  tlp::Graph *displayGraph;
  OpenCloseHierarchy *openCloseHierarchy;

  /**
   * Computes the bends for the passed edge.
   * \param curEdge The edge which we compute the bends for.
   */
  void computeBend (const tlp::edge &curEdge);

  /**
   * Function to compute all the bends from scratch.
   */
  void computeBends ();
};
#endif
