#include <tulip/DoubleProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/IntegerProperty.h>
#if (QT_REL == 3)
#include <qregexp.h>
#else
#include <QtCore/qregexp.h>
#include "tulip/Qt3ForTulip.h"
#endif

#include "FindSelection.h"

using namespace tlp;
using namespace std;

FindSelection::FindSelection (Graph *graph, PropertyInterface *property) :
  graph(graph), property(property) {}

static bool EvalProxy( PropertyInterface *p, const node &n,
		       std::string value, int mode) {
  DoubleProperty *    metp = dynamic_cast<DoubleProperty*>(p);
  StringProperty *    strp = dynamic_cast<StringProperty*>(p);
  BooleanProperty * selp = dynamic_cast<BooleanProperty*>(p);
  IntegerProperty * intp = dynamic_cast<IntegerProperty*>(p);
  
  if( metp ) {
    // DoubleProperty 
    DoubleType::RealType v = metp->getNodeValue( n );
    DoubleType::RealType v0 = atof( value.c_str() );
    if( mode == 0 )		return v < v0;
    if( mode == 1 )		return v <= v0;
    if( mode == 2 )		return v == v0;
    if( mode == 3 )		return v >= v0;
    if( mode == 4 )		return v > v0;
    else return v != v0;
  }
  
  if( strp ) {
    // StringProperty 
    std::string v = strp->getNodeValue( n );
    std::string v0 = value.c_str();
    QRegExp rexp( v0.c_str() );
    if( mode == 2 ) return rexp.exactMatch( v.c_str() );
    else return !rexp.exactMatch( v.c_str() );
  }
  
  if( selp ) {
    // BooleanProperty 
    bool v  = selp->getNodeValue( n );
    bool v0 = !( value.size() == 0 || value == "False" || value == "false" || value == "0" );
    if( mode == 2 ) return v == v0;
    else return v != v0;
  }

  if( intp ) {
    // IntegerProperty 
    IntegerType::RealType v = intp->getNodeValue( n );
    IntegerType::RealType v0 = atoi( value.c_str() );
    if( mode == 0 )		return v < v0;
    if( mode == 1 )		return v <= v0;
    if( mode == 2 )		return v == v0;
    if( mode == 3 )		return v >= v0;
    if( mode == 4 )		return v > v0;
    else return v != v0;
  }
  
  return false;
}

static std::string EvalBracketProxy( PropertyInterface *p, const node &n,
				     std::string value) {
  string stringValue = p->getNodeStringValue (n);
  QRegExp rexp(value.c_str());
  int matchPos = rexp.search(stringValue);
  if (matchPos < 0) return "()";
  string catMatch ("(");
  QStringList list = rexp.capturedTexts();
  QStringList::Iterator it = list.begin();
  ++it;
  while (it != list.end()) {
    catMatch += string ((*it).latin1()) + string(",");
    ++it;
  }//end while
  catMatch[catMatch.length() - 1] = ')';
  return catMatch;
}

void FindSelection::evalBracketNodes (const std::string &fv, 
				      StringProperty *bracketMatch) {
  Iterator<node> * nodeIt = graph->getNodes();
  while( nodeIt->hasNext() ) {
    node n = nodeIt->next();
    string matchCode = EvalBracketProxy(property, n, fv);
    bracketMatch->setNodeValue (n, matchCode);
  } delete nodeIt;
}//end evalBracketNodes

void FindSelection::evalNodes(const std::string &fv, BooleanProperty *selP) {
  Iterator<node> * nodeIt = graph->getNodes();
  while( nodeIt->hasNext() ) {
    node n = nodeIt->next();
    bool v = EvalProxy(property, n, fv, 2);
    selP->setNodeValue (n, v);
  } delete nodeIt;
}

void FindSelection::find (const string &searchString, BooleanProperty *selP) {
  evalNodes(searchString, selP);
}

void FindSelection::findCaptureText (const string &searchString,
				     StringProperty *bracketMatchProperty) {
  evalBracketNodes (searchString, bracketMatchProperty);
}

