#include <getopt.h>
#include <string>
#include <qapplication.h>
#include <tulip/Graph.h>
#include <tulip/TlpTools.h>
#include <tulip/GlGraph.h>
#include <tulip/GlGraphWidget.h>
#include "GrouseInterfaceBackEnd.h"

#define NO_PLUGINS_OPTION ('N')
static struct option grouse_options[] =
  {
    /* These options set a flag. */
    {"no-plugins", no_argument, NULL, NO_PLUGINS_OPTION},
    {0, 0, 0, 0}
  };
const string valid_options ("N");
bool no_plugins = false;

unsigned int parseArgs (int argc, char **argv) {
  unsigned int numOptions = 0;
  int option = getopt_long (argc, argv, valid_options.c_str(), 
			    grouse_options, NULL);
  while (option != EOF) {
    switch (option) {
    case NO_PLUGINS_OPTION:
      no_plugins = true;
      break;
    case '?':
    default:
      break;
    }//end switch
    ++numOptions;
    option = getopt_long (argc, argv, valid_options.c_str(), 
			  grouse_options, NULL);
  }//end while
  return numOptions;
}//end parseArgs

int main(int argc, char **argv) {
  QApplication mainApp (argc,argv);
  unsigned int numOptions = parseArgs (argc, argv);
  /****************************************************/
  tlp::initTulipLib();
  if (!no_plugins) {
    tlp::loadPlugins();
    GlGraph::loadPlugins();
  }//end if
  else cerr << "Plugins are not loaded." << endl;
  
  /****************************************************/
  GrouseInterfaceBackEnd grouseMainWin;
  
  mainApp.setMainWidget(&grouseMainWin);
  grouseMainWin.resize(640,640);
  grouseMainWin.show();

  if (argc - numOptions == 2) {
    Graph *cmdLineGraph = tlp::loadGraph(argv[argc - 1]);
    if (cmdLineGraph == NULL)
      cerr << "Could not open the file " << argv[argc - 1] << endl;
    else
      grouseMainWin.initInterface (cmdLineGraph);
  }//end if
  return mainApp.exec();
}
