#ifndef __GROUSE_SELECTION_PROPERTY__
#define __GROUSE_SELECTION_PROPERTY__
#include <vector>
#include <qobject.h>
#include <tulip/BooleanProperty.h>
#include <tulip/Graph.h>
#include <tulip/MutableContainer.h>
#include <tulip/GlGraphWidget.h>
#include "ColourShapeDefaults.h"
#include "OpenCloseHierarchy.h"

class GrouseSelectionProperty : public QObject {
  Q_OBJECT;
 public:
  GrouseSelectionProperty (GlGraphWidget *displayWidget,
			   tlp::Graph *displayGraph,
			   tlp::Graph *hierarchy,
			   OpenCloseHierarchy *openCloseHierarchy); 
  ~GrouseSelectionProperty();
  
  void clearSelection ();
  void setSelection (const std::vector<tlp::node> &nodes,
		     std::vector<GlABoundingCircle *> &augs);
  void setEdgeSelection (const std::vector<tlp::edge> &edges);
  void setMouseSelection (const vector<tlp::node> &nodes,
			  vector<GlABoundingCircle *> &augs,
			  const tlp::node &lastSelectedHierarchyNode);
  void invertModifySelection (const vector<tlp::node> &nodes,
				  vector<GlABoundingCircle *> &augs);
  void invertModifyEdgeSelection (const vector<edge> &edges);
  void invertModifyMouseSelection (const vector<tlp::node> &nodes,
				   vector<GlABoundingCircle *> &augs,
				   const tlp::node &lastSelectedHierarchyNode);
  void modifySelection (const std::vector<tlp::node> &nodes,
			std::vector<GlABoundingCircle *> &augs, bool setTrue);
  void modifyEdgeSelection (const std::vector<tlp::edge> &edges, bool setTrue);
  tlp::BooleanProperty &getHierarchySelection ();
  tlp::BooleanProperty &getVisibleSelection ();

 signals:
  void selectionChanged (tlp::BooleanProperty &newSelection,
			 const tlp::node &lastSelectedHierarchyNode);

 private:
  GlGraphWidget *displayWidget;
  //node selection is done on hierarchy nodes and not display nodes.
  //(unify with augmented display selection)
  tlp::BooleanProperty *nodeSelection; 
  //edge selection is done on the edges in the cut of the display graph.
  tlp::BooleanProperty *edgeSelection;
  tlp::BooleanProperty *visibleSelection;
  tlp::Graph *displayGraph;
  tlp::Graph *hierarchy;
  OpenCloseHierarchy *openCloseHierarchy;
  void setNodeSelection (const node &displayNode, bool value);
  void setEdgeSelection (const edge &displayEdge, bool value); 
  void setAugSelection (GlABoundingCircle *aug, bool value);
  void clearSelectionNoDraw ();
 
};

#endif
