#include <iostream>
#include <sstream>
#include <string>

#if (QT_REL == 3)
#include <qstring.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qcolordialog.h>
#include <qpainter.h>
#include <qfontmetrics.h>
#include <qapplication.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qvalidator.h>
#include <qsizepolicy.h>
#include <qmessagebox.h>
#include <qinputdialog.h>
#else
#include <QtCore/qstring.h>
#include <QtGui/qpushbutton.h>
#include <QtGui/qfiledialog.h>
#include <QtGui/qcolordialog.h>
#include <QtGui/qpainter.h>
#include <QtGui/qfontmetrics.h>
#include <QtGui/qapplication.h>
#include <QtGui/qlayout.h>
#include <QtGui/qlineedit.h>
#include <QtCore/qdir.h>
#include <QtCore/qfileinfo.h>
#include <QtGui/qvalidator.h>
#include <QtGui/qsizepolicy.h>
#include <QtGui/qmessagebox.h>
#include <QtGui/qinputdialog.h>
#include <QtGui/qevent.h>
#include "tulip/Qt3ForTulip.h"
#endif

#include <tulip/Graph.h>
#include <tulip/DoubleProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/BooleanProperty.h>
#include <tulip/LayoutProperty.h>
#include <tulip/IntegerProperty.h>
#include <tulip/ColorProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/GlGraph.h>
#include <tulip/Glyph.h>
#include <tulip/ForEach.h>

#include "GrouseTableWidget.h"

#define COLORTABLEITEM_RTTI 1001
#define FILETABLEITEM_RTTI  1002
#define SIZETABLEITEM_RTTI  1003
#define COORDTABLEITEM_RTTI 1004
#define GLYPHTABLEITEM_RTTI 1005
#define SELECTIONTABLEITEM_RTTI 1006
#define EDGESHAPETABLEITEM_RTTI 1007
#define LABELPOSITIONTABLEITEM_RTTI 1008

#define NO_ROW (0xffffffff)

using namespace std;
using namespace tlp;

//================================================================================
GrouseTableWidget::GrouseTableWidget(QWidget *parent, const char *name,
				     Qt::WFlags fl) :
  QTable(parent, name), updateColumnTitle(false) {
  resetBackColor1();
  resetBackColor2();
  this->setSelectionMode (QTable::NoSelection);
}

GrouseTableWidget::~GrouseTableWidget() {}

bool GrouseTableWidget::getUpdateColumnTitle() const {return updateColumnTitle;}
void GrouseTableWidget::setUpdateColumnTitle(const bool b) { updateColumnTitle = b;}

QColor GrouseTableWidget::getBackColor1() const {return backColor1;}
void GrouseTableWidget::setBackColor1(const QColor &c) {backColor1 = c;}
void GrouseTableWidget::resetBackColor1() {backColor1 = QColor(236, 245, 255);}

QColor GrouseTableWidget::getBackColor2() const {return backColor2;}
void GrouseTableWidget::setBackColor2(const QColor &c) {backColor2 = c;}
void GrouseTableWidget::resetBackColor2() {backColor2 = QColor(250, 250, 250);}

QColor GrouseTableWidget::backgroundColor(const int row) const {
  return (row % 2) ? backColor1 : backColor2;
}

void GrouseTableWidget::paintCell(QPainter * p, int row, int col, const QRect &cr, bool selected, const QColorGroup &cg) {
  QColorGroup g(cg);
  g.setColor(QColorGroup::Base, backgroundColor(row));
  QTable::paintCell(p, row, col, cr, selected, g);
}

//======================================================================
void GrouseTableWidget::addRow (const node &curNode) {
  char nodeId[10];
  int curRow = this->numRows ();
  this->insertRows (curRow, 1); 
  nodeIdToRow.set (curNode.id, curRow);
  sprintf (nodeId, "%d", curNode.id);
  this->verticalHeader()->setLabel (curRow, QString (nodeId));
}//end forEach

//======================================================================
void GrouseTableWidget::populateTableWithLeaves (Graph *curGraph,
						 GraphProperty 
						 *metaGraphHierarchy) {
  node curNode;
  forEach (curNode, curGraph->getNodes ()) {
    if (metaGraphHierarchy->getNodeValue (curNode) == NULL) {
      int rowNumber = nodeIdToRow.get (curNode.id);
      for (unsigned int i = 0; i < displayableProperties.size (); ++i) {
	string curPropName = displayableProperties[i];
	PropertyInterface *curProperty = curGraph->getProperty (curPropName);
	this->setTulipNodeItem(curProperty, curPropName, curNode,
			       rowNumber, i);
      }//end for i
    }//end if
    else
      populateTableWithLeaves (metaGraphHierarchy->getNodeValue (curNode),
			       metaGraphHierarchy);
  }//end forEach
}//end populateTableWithLeaves
//======================================================================
void GrouseTableWidget::addProperty (std::string propertyName,
				     Graph *curGraph) {
  int numCol = this->numCols ();
  this->insertColumns (numCol, 1); 
  this->setItem (0, numCol, new SelectionTableItem (this, ""));
  this->horizontalHeader()->setLabel (numCol, QString (propertyName));
  this->setColumnReadOnly (numCol, FALSE);
}//end addProperty

//======================================================================
void GrouseTableWidget::numLeavesAndRowHash (Graph *curHierarchy,
					     GraphProperty *metaGraphHierarchy,
					     unsigned int &nextLeafRow) {
  node curNode;
  forEach (curNode, curHierarchy->getNodes ()) {
    if (metaGraphHierarchy->getNodeValue (curNode) == NULL) {
      ++nextLeafRow;
      nodeIdToRow.set (curNode.id, nextLeafRow);
    }//end if
    else
      numLeavesAndRowHash (metaGraphHierarchy->getNodeValue (curNode),
			   metaGraphHierarchy,
			   nextLeafRow);
  }//end forEach
}

//======================================================================
void GrouseTableWidget::resetProperties (Graph *curHierarchy,
					 GraphProperty *metaGraphHierarchy) {
  nodeIdToRow.setAll (NO_ROW);
  displayableProperties.clear ();
  unsigned int numLeaves = 0;
  numLeavesAndRowHash (curHierarchy, metaGraphHierarchy, numLeaves);
  this->setNumCols (0);
  this->setNumRows (1 + numLeaves);
  this->setColumnMovingEnabled (TRUE);
  
  Iterator<string> * propIt = curHierarchy->getProperties();
  while (propIt->hasNext()) {
    string curPropName = propIt->next();
    PropertyInterface *curProperty = curHierarchy->getProperty (curPropName);
    string propertyTypename = PropertyInterface::getTypename (curProperty);
    if ((curPropName.find ("view") == string::npos) && 
	(curPropName.find ("Original Edge ID") == string::npos) &&
	((propertyTypename == "string") || (propertyTypename == "double") ||
	 (propertyTypename == "boolean") || (propertyTypename == "int"))) {
      displayableProperties.push_back (curPropName);
      this->addProperty (curPropName, curHierarchy);
    }//end if
  } delete propIt;
  populateTableWithLeaves (curHierarchy, metaGraphHierarchy);
}//end resetProperties
  
//======================================================================
void GrouseTableWidget::columnIndexChanged (int section, int fromIndex, 
					    int toIndex) {
  emit propertyReorderSignal (fromIndex, toIndex);
}

//======================================================================
void GrouseTableWidget::setTulipNodeItem(const PropertyInterface *editedProperty, const std::string propertyName, const node &n,
                                    const int row, const int col) {
  QString label;
  if (propertyName == "viewShape") {
    GlyphTableItem *item = new GlyphTableItem(this, false);
    int shapenum = ((IntegerProperty *)editedProperty)->getNodeValue(n);

    label = tr("Shape");
    this->setItem(row, col, item);
    item->setCurrentItem(GlGraph::glyphName(shapenum).c_str());
  } else if (propertyName == "viewLabelPosition") {
    LabelPositionTableItem *item = new LabelPositionTableItem(this, false);
    int labelPos = ((IntegerProperty *)editedProperty)->getNodeValue(n);

    label = tr("Label");
    this->setItem(row, col, item);
    item->setCurrentItem(GlGraph::labelPositionName(labelPos).c_str());
  }
  else if (propertyName == "viewTexture") {
    FileTableItem *item = new FileTableItem(this);
    item->setText( const_cast<PropertyInterface *>(editedProperty)->getNodeStringValue(n).c_str() );
    label = tr("Texture");
    this->setItem(row, col, item);
  }
  else if (typeid(*editedProperty) == typeid(BooleanProperty)) {
    BooleanProperty *tmpSel = (BooleanProperty *) editedProperty;
    SelectionTableItem *item = new SelectionTableItem(this, "");
    item->setChecked(tmpSel->getNodeValue(n));
    label = tr("Selected");
    this->setItem(row, col, item);
  }
  else if (typeid(*editedProperty) == typeid(ColorProperty)) {
    ColorProperty *tmpCol = (ColorProperty *)editedProperty;
    Color c = tmpCol->getNodeValue(n);
    ColorTableItem *item = new ColorTableItem(this, qRgba(c[0], c[1], c[2], c[3]));
    label = tr("Color");
    this->setItem(row, col, item);
  }
  else if (typeid(*editedProperty) == typeid(SizeProperty)) {
    SizeProperty *tmpSiz = (SizeProperty *) editedProperty;
    Size s = tmpSiz->getNodeValue(n);
    SizeTableItem *item = new SizeTableItem(this);
    item->setSize(s);
    label = tr("Size");
    this->setItem(row, col, item);
  }
  else if (typeid(*editedProperty) == typeid(LayoutProperty)) {
    LayoutProperty *tmpLay = (LayoutProperty *)editedProperty;
    Coord c = tmpLay->getNodeValue(n);
    CoordTableItem *item = new CoordTableItem(this);
    item->setCoord(c);
    label = tr("Coord");
    this->setItem(row, col, item);
  }
  else {
    QTableItem *item = new QTableItem(this, QTableItem::OnTyping,
                                      const_cast<PropertyInterface *>(editedProperty)->getNodeStringValue(n).c_str() );
    label = tr("Value");
    this->setItem(row, col, item);
  }

  if (updateColumnTitle) {
    this->horizontalHeader()->setLabel(col, label);
  }
}

void GrouseTableWidget::setTulipEdgeItem(const PropertyInterface *editedProperty, const std::string propertyName, const edge &e,
                                    const int row, const int col) {
  QString label;
  if (typeid(*editedProperty) == typeid(BooleanProperty)) {
    BooleanProperty *tmpSel = (BooleanProperty *)editedProperty;
    SelectionTableItem *item = new SelectionTableItem(this, "");
    item->setChecked(tmpSel->getEdgeValue(e));
    label = tr("Selected");
    this->setItem(row, col, item);
  }
  else if (typeid(*editedProperty) == typeid(ColorProperty)) {
    ColorProperty *tmpCol = (ColorProperty *)editedProperty;
    Color c = tmpCol->getEdgeValue(e);
    ColorTableItem *item = new ColorTableItem(this, qRgba(c[0], c[1], c[2], c[3]));
    label = tr("Color");
    this->setItem(row, col, item);
  }
  else if (typeid(*editedProperty) == typeid(SizeProperty)) {
    SizeProperty *tmpSiz = (SizeProperty *)editedProperty;
    Size s = tmpSiz->getEdgeValue(e);
    SizeTableItem *item = new SizeTableItem(this);
    item->setSize(s);
    label = tr("Size");
    this->setItem(row, col, item);
  } else if (propertyName == "viewShape") {
    EdgeShapeTableItem *item = new EdgeShapeTableItem(this, false);
    int shapenum = ((IntegerProperty *)editedProperty)->getEdgeValue(e);

    label = tr("Shape");
    this->setItem(row, col, item);
    item->setCurrentItem(GlGraph::edgeShapeName(shapenum).c_str());
  }
//   else if (typeid(*editedProperty) == typeid(Layout)) {
//     Layout *tmpLay = getProperty<LayoutProperty>(graph, propertyName);
//     Coord c = tmpLay->getEdgeValue(e);
//     CoordTableItem *item = new CoordTableItem(this);
//     item->setCoord(c);
//     label = tr("Coord");
//     this->setItem(row, col, item);
//   }
  else {
    QTableItem *item = new QTableItem(this, QTableItem::OnTyping,
                                      const_cast<PropertyInterface *>(editedProperty)->getEdgeStringValue(e).c_str() );
    label = tr("Value");
    this->setItem(row, col, item);
  }

  if (updateColumnTitle) {this->horizontalHeader()->setLabel(col, label);}
}
