#ifndef GROUSE_TABLE_WIDGET_H
#define GROUSE_TABLE_WIDGET_H

#if (__GNUC__ < 3)
#include <hash_map>
#else
#include <ext/hash_map>
#endif
#include <string>

#if (QT_REL == 3)
#include <qtable.h>
#include <qbutton.h>
#include <qpainter.h>
#else
#include <Qt3Support/q3button.h>
#include <Qt/qpainter.h>
#include <Qt3Support/q3table.h>
#include "tulip/Qt3ForTulip.h"
#endif


#include <tulip/Graph.h>
#include <tulip/Size.h>
#include <tulip/Coord.h>
#include <tulip/MutableContainer.h>
#include <tulip/GraphProperty.h>
#include <tulip/TulipTableWidget.h>

/** \addtogroup Tulip_Widgets */ 
/*@{*/
/* this table handle displaying and editing node or edge properties */
class GrouseTableWidget : public QTable {
  Q_OBJECT;
  Q_PROPERTY(QColor backColor1 READ getBackColor1 WRITE setBackColor1 RESET resetBackColor1);
  Q_PROPERTY(QColor backColor2 READ getBackColor2 WRITE setBackColor2 RESET resetBackColor2);
  Q_PROPERTY(bool updateColumnTitle READ getUpdateColumnTitle WRITE setUpdateColumnTitle);
  
 private:
  tlp::MutableContainer<int> nodeIdToRow;
  std::vector<std::string> displayableProperties;
  
protected:
  QColor backColor1;
  QColor backColor2;
  bool updateColumnTitle;

 public:
  GrouseTableWidget(QWidget *parent = 0, const char * name = 0, Qt::WFlags fl = 0);
  ~GrouseTableWidget();
  
  inline int getRow (const tlp::node &curNode) { 
    return nodeIdToRow.get (curNode.id); 
  }
  void addRow (const tlp::node &curNode);
  void addProperty (std::string propertyName, tlp::Graph *curGraph);
  void resetProperties (tlp::Graph *curHierarchy,
			tlp::GraphProperty *metaGraphHierarchy);
  void populateTableWithLeaves (tlp::Graph *curGraph,
				tlp::GraphProperty *metaGraphHierarchy);
  void numLeavesAndRowHash (tlp::Graph *curHierarchy,
			    tlp::GraphProperty *metaGraphHierarchy,
			    unsigned int &nextLeafRow);
  void setTulipNodeItem(const tlp::PropertyInterface *, const std::string propertyName, const tlp::node &, const int row, const int col);
  QString getTulipNodeValue(const int row, const int col) const;
  void setTulipEdgeItem(const tlp::PropertyInterface *, const std::string propertyName, const tlp::edge &, const int row, const int col);
  QString getTulipEdgeValue(const int row, const int col) const;
  QColor getBackColor1() const;
  QColor getBackColor2() const;
  bool getUpdateColumnTitle() const;
  QColor backgroundColor(const int row) const;
  void paintCell(QPainter *, int row, int col, const QRect &, bool selected, const QColorGroup &);
  
  public slots:
    void setBackColor1(const QColor &);
  void resetBackColor1();
  void setBackColor2(const QColor &);
  void resetBackColor2();
  void setUpdateColumnTitle(const bool);

  protected slots:
    void columnIndexChanged (int section, int fromIndex, int toIndex);

  signals:
    void propertyReorderSignal (int fromIndex, int toIndex);
};

/*@}*/
#endif //PROPERTYWIDGETS_H
