#include "HSVConversion.h"
using namespace tlp;

//===================================================================
#define UNDEFINED (0) 
HSV rgb_to_hsv (const Color &rgb) {
  float R = rgb.getR()/255.0;
  float G = rgb.getG()/255.0;
  float B = rgb.getB()/255.0;
  float v, x, f;
  int i;
  x = R; v = R;
  if (x > G) x = G;
  if (x > B) x = B;
  if (v < G) v = G;
  if (v < B) v = B;
  if(v == x) return HSV(UNDEFINED, 0, v);
  f = (fabs(R - x) < 1e-6) ? G - B : (fabs (G - x) < 1e-6 ? B - R : R - G);
  i = (fabs(R - x) < 1e-6) ? 3 : (fabs(G - x) < 1e-6 ? 5 : 1);
  return HSV(i - f /(v - x), (v - x)/v, v);
}

//===================================================================
inline unsigned char toChar (float value) {
  return static_cast<unsigned char> (floor (value * 255));
}

//===================================================================
Color hsv_to_rgb (const HSV &hsv) {

  // H is given on [0, 6] or UNDEFINED. S and V are given on [0, 1].
  // RGB are each returned on [0, 1].
  float h = hsv.h, s = hsv.s, v = hsv.v, m, n, f;
  int i;
  if(h == UNDEFINED) return Color (toChar(v), toChar(v), toChar(v));

  i = floor(h);
  f = h - i;
  if(!(i & 1)) f = 1 - f; // if i is even
  m = v * (1 - s);
  n = v * (1 - s * f);
  switch (i) {
  case 6:
  case 0: return Color(toChar(v), toChar(n), toChar(m));
  case 1: return Color(toChar(n), toChar(v), toChar(m));
  case 2: return Color(toChar(m), toChar(v), toChar(n));
  case 3: return Color(toChar(m), toChar(n), toChar(v));
  case 4: return Color(toChar(n), toChar(m), toChar(v));
  case 5: return Color(toChar(v), toChar(m), toChar(n));
  }
}
