#include <tulip/StableIterator.h>
#include <tulip/MutableContainer.h>
#include <tulip/StringProperty.h>
#include <tulip/DoubleProperty.h>
#include "HierarchyMap.h"
#include "HierarchyNodeCreation.h"

using namespace tlp;
using namespace std;

//========================================================================
//a function that copies the nodes and edges from src to dest
void copy (Graph *src, Graph *dest) {
  StableIterator <node> nodes (src->getNodes());
  while (nodes.hasNext()) dest->addNode (nodes.next());
  StableIterator <edge> edges (src->getEdges());
  while (edges.hasNext()) dest->addEdge (edges.next());
}//end copyFeature

//========================================================================
//a function that deletes feature from delGraph
void delFeature (Graph *feature, Graph *delGraph) {
  StableIterator <node> nodes (feature->getNodes());
  while (nodes.hasNext ()) delGraph->delNode (nodes.next());
}//end deleteFeatureGraphFromWorking

//========================================================================
void mergeOrigEdges (const edge &curEdge,
		     const edge &createdEdge,
		     StringProperty *originalEdges) {
  if (originalEdges->getEdgeValue (curEdge) != "") {
    if (originalEdges->getEdgeValue (createdEdge) != "")
      originalEdges->setEdgeValue (createdEdge,
				   originalEdges->getEdgeValue (createdEdge) 
				   + " " +
				   originalEdges->getEdgeValue 
				   (curEdge));
    
    else 
      originalEdges->setEdgeValue (createdEdge,
				   originalEdges->getEdgeValue (curEdge));
  }//end if
}

//========================================================================
void setNewConnections (Graph *hierarchySubGraph, Graph *newSubGraph,
			const node &newClusterNode) {
  //a mutable container to indicate when an edge has already been
  //drawn.
  MutableContainer <bool> edgeDone;
  MutableContainer <edge> createdEdge;
  edgeDone.setAll (false);
  StringProperty *originalEdges = 
    hierarchySubGraph->getProperty <StringProperty> (ORIGINAL_EDGE_NAME);
  //for all the nodes in the newSubGraph
  StableIterator<node> nodes (newSubGraph->getNodes());
  while (nodes.hasNext ()) {
    node curNode = nodes.next();
   
    //copy over the edges
    StableIterator<edge> edges (hierarchySubGraph->getInOutEdges (curNode));
    while (edges.hasNext()) {
      edge curEdge = edges.next();
      if (!newSubGraph->isElement (curEdge)) {
	node opposite = hierarchySubGraph->opposite (curEdge, curNode);
	if (!edgeDone.get (opposite.id)) {
	  assert(opposite != curNode);
	  edge created = hierarchySubGraph->addEdge (newClusterNode, opposite);
	  edgeDone.set (opposite.id, true);
	  createdEdge.set (opposite.id, created);
	  originalEdges->setEdgeValue (created, "");
	}//end if
	mergeOrigEdges (curEdge, createdEdge.get (opposite.id), 
			originalEdges);
	hierarchySubGraph->delEdge (curEdge);
      }//end if
    }//end for i
  }//end while
}//end setFeatureNodeConnections

//================================================================
bool processConnectedComponents (Graph *curHierarchyGraph,
				 Graph *components,
				 vector<node> &addedHierarchyNodes,
				 Graph *hierarchy,
				 Graph *hierarchySubGraphs,
				 GraphProperty *hierarchyMetaGraphs,
				 const string &errorLocation) {

  //decompose subgraphs into connected components
  DoubleProperty connectedComponents (components);
  string errMsg;
  bool success = components->computeProperty ("Connected Component", 
					      &connectedComponents, errMsg);
  if (!success) {
    cerr << "Connected components returned the following error " << endl;
    cerr << errMsg << endl << " is the plugin installed?" << endl;
    cerr << errorLocation << endl;
    return false;
  }//end if
  
  DataSet localData;
  localData.set("Property", &connectedComponents);
  success = tlp::applyAlgorithm (components, errMsg, &localData, 
				 "Equal Value");
  if (!success) {
    cerr << "Equal Value clustering returned the following error:\n"
	 << errMsg << "\n(Is the plugin properly installed?)" << endl;
    cerr << errorLocation << endl;
    return false;
  }//end if
  
  StableIterator<Graph *> concomps (components->getSubGraphs());
  while (concomps.hasNext()) {
    Graph *curConComp = concomps.next();
    if (curConComp->numberOfNodes() > 1) {
      Graph *newHierarchyGraph = hierarchySubGraphs->addSubGraph();
      copy (curConComp, newHierarchyGraph);
      node newHierarchyNode = curHierarchyGraph->addNode();
      addedHierarchyNodes.push_back (newHierarchyNode);
      hierarchyMetaGraphs->setNodeValue (newHierarchyNode, newHierarchyGraph);
      setNewConnections (curHierarchyGraph, 
			 newHierarchyGraph, newHierarchyNode);
      delFeature (newHierarchyGraph, curHierarchyGraph);
    }//end if
    else addedHierarchyNodes.push_back (curConComp->getOneNode ());
  }//end while
  return true;
}//end processConnectedComponents
