#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if (QT_REL == 3)
#include <qevent.h>
#else
#ifdef  _WIN32
// compilation pb workaround
#include <windows.h>
#endif
#include <QtGui/qevent.h>
#include "tulip/Qt3ForTulip.h"
#endif

#include <tulip/Graph.h>
#include <tulip/StableIterator.h>
#include <tulip/BooleanProperty.h>
#include "MouseFlowSelection.h"
#include "MetaNodeBoundingCircle.h"

#define MAX_UINT (0xffffffff)

using namespace std;
using namespace tlp;
using namespace stdext;

#define DEFAULT_MAX_PATH (20)
#define QT_SPAN_TREE_KEY_DOWN (Qt::ShiftButton)
#define QT_PATHS_KEY_DOWN (Qt::ControlButton)
int MouseFlowSelection::maxPath = DEFAULT_MAX_PATH;

//======================
MouseFlowSelection::MouseFlowSelection (GrouseSelectionProperty *grouseSelectionProperty, OpenCloseHierarchy *openCloseHierarchy) :
  spanTreeStart (false), pathStart (false),
  hopNumber(0), bfsDistanceQueue (NULL), curPathPosition (0),
  grouseSelectionProperty (grouseSelectionProperty),
  openCloseHierarchy (openCloseHierarchy) {
  nodeSelectedOnHopNumber.setAll (MAX_UINT);
}

//========================
bool MouseFlowSelection::eventFilter(QObject* widget, QEvent* e){
  GlGraphWidget *glGraphWidget = (GlGraphWidget *) widget;
  if (e->type() == QEvent::Wheel)
    return wheelEvent (glGraphWidget, static_cast<QWheelEvent *> (e));
  if (e->type() == QEvent::KeyRelease)
    return keyReleaseEvent (glGraphWidget, static_cast<QKeyEvent *> (e));
  return false;
}

//=========================
void MouseFlowSelection::updatePathSelection (Graph *graph) {
  const Path &reportPath = pathHistory[curPathPosition];
  vector<node> sortedPath (reportPath.pathElements.size ());
  vector<edge> edgeSortedPath (reportPath.pathElements.size () - 1);
  std::vector<GlABoundingCircle *> visibleSelectionAugMod;
  hash_map<node, unsigned int>::const_iterator reportPathIt = reportPath.pathElements.begin ();
  for (; reportPathIt != reportPath.pathElements.end (); ++reportPathIt)
    sortedPath[reportPathIt->second] = reportPathIt->first;
  for (unsigned int i = 0; i < sortedPath.size () - 1; ++i) {
    edgeSortedPath[i] = graph->existEdge (sortedPath[i], sortedPath[i + 1]);
    if (!edgeSortedPath[i].isValid ())
      edgeSortedPath[i] = graph->existEdge (sortedPath[i + 1], sortedPath[i]);
  }//end for
  grouseSelectionProperty->clearSelection ();
  grouseSelectionProperty->setSelection (sortedPath,
					 visibleSelectionAugMod);
  grouseSelectionProperty->setEdgeSelection (edgeSortedPath);
}//end updatePathSelection

//=========================
bool MouseFlowSelection::nextShortestPath (Graph *graph) {
  while (!bfsDistanceQueue->empty () && 
	 (bfsDistanceQueue->front ().curNode != end) &&
	 (bfsDistanceQueue->front ().curDistance < maxPath)) {
    Path curPath = bfsDistanceQueue->front ();
    bfsDistanceQueue->pop ();
    node opposite;
    forEach (opposite, graph->getInOutNodes (curPath.curNode)) {
      if (curPath.pathElements.find (opposite) == curPath.pathElements.end ()){
	Path nextPath;
	nextPath.curNode = opposite;
	nextPath.curDistance = curPath.curDistance + 1;
	nextPath.pathElements = curPath.pathElements;
	nextPath.pathElements[opposite] = nextPath.curDistance;
	bfsDistanceQueue->push (nextPath);
      }//end if
    }//end forEach
  }//end while
  
  if (!bfsDistanceQueue->empty ()) {
    pathHistory.push_back (bfsDistanceQueue->front ());
    bfsDistanceQueue->pop ();
    return true;
  }//end if
  else if (bfsDistanceQueue->front ().curDistance >= maxPath) {
    cerr << "Maximum depth of the path search tree exceeded.  " << endl
	 << maxPath << " currently at " 
	 << bfsDistanceQueue->front ().curDistance << endl;
    return false;
  }//end if

  cerr << "There are no more paths between these two vertices. " << endl;
  return false;
}//end while

//=========================
void MouseFlowSelection::oneHop (Graph *graph, bool forward) {
  vector<node> visibleSelectionNodeMod;
  vector<edge> visibleSelectionEdgeMod;
  vector<GlABoundingCircle *> visibleSelectionAugMod;
  bool proceed = false;
  StableIterator<unsigned int> curHopNodes (nodeSelectedOnHopNumber.findAll (hopNumber));
  while (curHopNodes.hasNext ()) {
    node lastSelectedDisplayNode (curHopNodes.next ());
    if (!lastSelectedDisplayNode.isValid ()) continue;
    if (!forward) {
      proceed = true;
      visibleSelectionNodeMod.push_back (lastSelectedDisplayNode);
      nodeSelectedOnHopNumber.set (lastSelectedDisplayNode.id, MAX_UINT);
      edge adjEdge;
      forEach (adjEdge, graph->getInOutEdges (lastSelectedDisplayNode)) {
	node opposite = graph->opposite (adjEdge, lastSelectedDisplayNode);
	if (nodeSelectedOnHopNumber.get (opposite.id) == hopNumber - 1)
	  visibleSelectionEdgeMod.push_back (adjEdge);
      }//end forEach
    }//end if
    else {
      edge adjEdge;
      forEach (adjEdge, graph->getInOutEdges (lastSelectedDisplayNode)) {
	node opposite = graph->opposite (adjEdge, lastSelectedDisplayNode);
	if (nodeSelectedOnHopNumber.get (opposite.id) == MAX_UINT) {
	  proceed = true;
	  visibleSelectionNodeMod.push_back (opposite);
	  visibleSelectionEdgeMod.push_back (adjEdge);
	  nodeSelectedOnHopNumber.set (opposite.id, hopNumber + 1);
	  //set edges to true/false
	}//end if
      } //end forEach
    }//end else
  }//end while
  grouseSelectionProperty->modifySelection (visibleSelectionNodeMod,
					    visibleSelectionAugMod,
					    forward);
  grouseSelectionProperty->modifyEdgeSelection (visibleSelectionEdgeMod,
						forward);
  if (proceed) {
    if (forward) ++hopNumber;
    else --hopNumber;
  }//end proceed
}//end oneHopForward

//=========================
bool MouseFlowSelection::wheelEvent(GlGraphWidget *glGraphWidget,QWheelEvent *qWheelEv){

  Graph *graph = glGraphWidget->getGraph ();
  if (qWheelEv->state () == QT_SPAN_TREE_KEY_DOWN) {
    if (!spanTreeStart) {
      spanTreeStart = true;
      hopNumber = 0;
      nodeSelectedOnHopNumber.setAll (MAX_UINT);
      BooleanProperty &selectionG = 
	grouseSelectionProperty->getVisibleSelection ();
      node curNode;
      forEach (curNode, selectionG.getNodesEqualTo (true)) {
	node displayNode = openCloseHierarchy->getDisplayNode (curNode);
	nodeSelectedOnHopNumber.set (displayNode.id, 0);
      }//end forEach
    }//end if
    
    //forwards expand out a hop, backwards retract in.  Zero do nothing
    if (qWheelEv->delta () == 0) {
      qWheelEv->ignore ();
      return false;
    }//end if

    if (qWheelEv->delta () > 0) oneHop (graph, true);
    else if (hopNumber > 0) oneHop (graph, false);
    return true;
  }//end if 

  else if (qWheelEv->state () == QT_PATHS_KEY_DOWN) {
    if (!pathStart) {
      pathStart = true;
      curPathPosition = 0;
      pathHistory.clear ();
      if (bfsDistanceQueue != NULL) delete bfsDistanceQueue;
      bfsDistanceQueue = new queue<Path> ();
      BooleanProperty &selectionG = 
	grouseSelectionProperty->getVisibleSelection ();
      node curNode;
      unsigned int counter = 0;
      forEach (curNode, selectionG.getNodesEqualTo (true)) {
	if (counter == 0) 
	  start = openCloseHierarchy->getDisplayNode (curNode);
	else if (counter == 1) 
	  end = openCloseHierarchy->getDisplayNode (curNode);
	else {
	  cerr << "Selection needs to be two elements. " << endl;
	  return false;
	}//end else
	++counter;
      }//end forEach
      if (graph->deg (start) > graph->deg (end)) {
	node swap = start;
	start = end;
	end = swap;
      }//end if
      Path startFrame;
      startFrame.curDistance = 0;
      startFrame.curNode = start;
      startFrame.pathElements[start] = 0;
      bfsDistanceQueue->push (startFrame);
    }//end if

    if ((qWheelEv->delta () > 0) && (curPathPosition < pathHistory.size()))  
      ++curPathPosition;
    else if (curPathPosition > 0) 
      --curPathPosition;
   
    if (curPathPosition >= pathHistory.size ()) {
      bool hasNewPath = nextShortestPath (graph);
      if (!hasNewPath) --curPathPosition;
    }//end if
    updatePathSelection (graph);
    return true;
  }//end else if
  qWheelEv->ignore ();
  return false;
}

//=====================================
bool MouseFlowSelection::keyReleaseEvent(GlGraphWidget *glGraphWidget, QKeyEvent *e) {
  if(e->key() == Qt::Key_Shift) {
    spanTreeStart = false;
    return true;
  }
  else if (e->key() == Qt::Key_Control) {
    pathStart = false;
    if (bfsDistanceQueue != NULL) delete bfsDistanceQueue;
    bfsDistanceQueue = NULL;
    return true;
  }//end else if

  e->ignore ();
  return false;
}
