//-*-c++-*-
#ifndef MOUSEFLOWSELECTION_H
#define MOUSEFLOWSELECTION_H
#if (__GNUC__ < 3)
#  include <hash_map>
#else
#  include <ext/hash_map>
#endif
#include <vector>
#include <queue>
#include <qobject.h>
#include <qevent.h>
#include <tulip/GWInteractor.h>
#include <tulip/Graph.h>
#include <tulip/MutableContainer.h>
#include "OpenCloseHierarchy.h"
#include "GrouseSelectionProperty.h"

class MouseFlowSelection: public  tlp::GWInteractor
{

private:
  class Path {
  public:
    unsigned int curDistance;
    tlp::node curNode;
    stdext::hash_map<tlp::node, unsigned int> pathElements;
  };


  bool spanTreeStart, pathStart;
  static int maxPath;
  node start, end;
  unsigned int hopNumber;
  tlp::MutableContainer<unsigned int> nodeSelectedOnHopNumber;
  unsigned int curPathPosition;
  std::queue<Path> *bfsDistanceQueue;
  std::vector<Path> pathHistory;
  GrouseSelectionProperty *grouseSelectionProperty;
  OpenCloseHierarchy *openCloseHierarchy;

  bool nextShortestPath (tlp::Graph *graph);
  void updatePathSelection (tlp::Graph *graph);
  void oneHop (tlp::Graph *graph, bool forward);
  bool keyReleaseEvent (GlGraphWidget *glGraphWidget, QKeyEvent *e);
  bool wheelEvent(GlGraphWidget *, QWheelEvent *);
  
public:
  MouseFlowSelection(GrouseSelectionProperty *grouseSelectionProperty,
		     OpenCloseHierarchy *openCloseHierarchy);
  inline ~MouseFlowSelection() {
    if (bfsDistanceQueue != NULL) delete bfsDistanceQueue;
  }
  
  inline static int getMaxPath () { return maxPath; }
  inline static void setMaxPath (int newMaxPath) {
    maxPath = newMaxPath;
  }//end setMaxPath

  bool eventFilter(QObject *widget, QEvent* e);
  GWInteractor * clone() { return new MouseFlowSelection(grouseSelectionProperty, openCloseHierarchy);}  
};

#endif
