#include <tulip/BooleanProperty.h>
#include "MouseGrouseGeneralOperations.h"
#include "MetaNodeBoundingCircle.h"

using namespace tlp;
using namespace std;

//===============================================================
//This local function picks the top augmented display
GlAugmentedDisplay *pickTopCircle (vector <pair <string, GlAugmentedDisplay *> > &pickedAug) {
  vector <pair <string, GlAugmentedDisplay *> >::iterator pickedAugIt =
    pickedAug.begin();
  GlABoundingCircle *curCircle = 
    static_cast<GlABoundingCircle *> (pickedAugIt->second);
  GlABoundingCircle *topCircle = curCircle;
  double topDepth = curCircle->getDepth();
  for (++pickedAugIt; pickedAugIt != pickedAug.end(); ++pickedAugIt) {
    curCircle = static_cast<GlABoundingCircle *> (pickedAugIt->second);
    if (topDepth < curCircle->getDepth()) {
      topCircle = curCircle;
      topDepth = curCircle->getDepth();
    }//end if
  }//end for
  return topCircle;
}//end pickTopCircle

//===============================================================
void pick (unsigned int x, unsigned int y, GlGraphWidget *glW, bool reset,
	   bool resetAfter,
	   bool pickNodes, bool pickEdges, bool pickAugs,
	   node &topNode, edge &topEdge, GlAugmentedDisplay *&topAug) {
  if (!(pickNodes || pickEdges || pickAugs)) return;
  Observable::holdObservers();
  bool result;
  ElementType type;

  node tmpNode;
  edge tmpEdge;
  BooleanProperty *elementSelected = 
    glW->getGraph()->getProperty<BooleanProperty>("viewSelection");
  if (reset) {
    elementSelected->setAllNodeValue(false);
    elementSelected->setAllEdgeValue(false);
  }
  
  if (pickNodes || pickEdges) {
    result = glW->doSelect(x, y, type, tmpNode, tmpEdge);
    if (result) {
      switch(type) {
      case NODE:
	if (pickNodes) {
	  elementSelected
	    ->setNodeValue(tmpNode, !elementSelected->getNodeValue(tmpNode));
	  topNode = tmpNode;
	}
	break;
      case EDGE:
	if (pickEdges) {
	  elementSelected
	    ->setEdgeValue(tmpEdge, !elementSelected->getEdgeValue(tmpEdge));
	  topEdge = tmpEdge;
	}
	break;
      }
    }
  }//end if

  if (pickAugs) {
    vector <pair <string, GlAugmentedDisplay *> > pickedAugmentedDisplays;
    result = glW->doSelectAugmentedDisplay (x, y, pickedAugmentedDisplays);
    if (result) topAug = pickTopCircle (pickedAugmentedDisplays);
    else topAug = NULL;
  }
  if (resetAfter) {
    elementSelected->setAllNodeValue(false);
    elementSelected->setAllEdgeValue(false);
  }
  Observable::unholdObservers();
}
