#ifndef __MOUSE_GROUSE_GENERAL_OPERATIONS__
#define __MOUSE_GROUSE_GENERAL_OPERATIONS__

/**
 * The following file implements a general picking interface with Tulip.
 * It is generally used with the mouse interactors of grouse.
 */

#include <tulip/GlGraphWidget.h>
#include <tulip/Graph.h>

/**
 * Function to pick a node, edge, or augmented display.
 * \param x The x coordinate of the mouse
 * \param y The y coordinate of the mouse
 * \param reset reset view selection before proceeding with pick
 * \param resetAfter reset view selection after proceeding with pick
 * \param glW The GlGraphWidget in which the mouse was clicked.
 * \param pickNodes Set this value to true if you want to pick nodes.
 * \param pickEdges Set this value to true if you want to pick edges.
 * \param pickAugs Set this value to true if you want to pick augmented 
 * displays
 * \param topNode The topmost node which was picked.
 * \param topEdge The topmost edge which was picked.
 * \param topAug The topmost augmented display which was picked.
 */
void pick (unsigned int x, unsigned int y, GlGraphWidget *glW, bool reset,
	   bool resetAfter,
	   bool pickNodes, bool pickEdges, bool pickAugs,
	   tlp::node &topNode, tlp::edge &topEdge, GlAugmentedDisplay *&topAug);
#endif
