#include <qcursor.h>
#include <tulip/BooleanProperty.h>
#include "MouseGrouseHierarchyNavigate.h"
#include "MouseGrouseGeneralOperations.h"

using namespace tlp;
using namespace std;

//===============================================================
//Defines for buttons to be used on each mouse operation
#define QT_OPEN_CLOSE_BUTTON (Qt::RightButton)
#define QT_OPEN_CLOSE_KEY_DOWN (Qt::NoButton)
#define QT_POWER_OPEN_BUTTON (Qt::RightButton)
#define QT_POWER_OPEN_KEY_DOWN (Qt::ShiftButton|Qt::ControlButton)
#define QT_CLEAR_LAYOUT_BUTTON (Qt::RightButton)
#define QT_CLEAR_LAYOUT_KEY_DOWN (Qt::ShiftButton)

//===============================================================
/*
 * Default constructors and destructors
 */
MouseGrouseHierarchyNavigate::MouseGrouseHierarchyNavigate (OpenCloseHierarchy 
					  *openCloseHierarchy,
					  MetaGraphHierarchyWidget 
					  *metaGraphHierarchy,
					  GrouseMorphing *grouseMorph) :
  openCloseHierarchy (openCloseHierarchy), 
  metaGraphHierarchy(metaGraphHierarchy),
  grouseMorph (grouseMorph) {}
MouseGrouseHierarchyNavigate::~MouseGrouseHierarchyNavigate() {}

//===============================================================
/**
 * Function to open/close nodes in the hierarchy and steer computation
 */
void MouseGrouseHierarchyNavigate::openClose (const QPoint &mouseClick,
					      GlGraphWidget *graphWidget,
					      const bool layoutReset) {
  node pickedNode;
  edge pickedEdge;
  GlAugmentedDisplay *pickedAug = NULL;
  pick (mouseClick.x(), mouseClick.y(), graphWidget, true, true, true, 
	false, true, pickedNode, pickedEdge, pickedAug);

  node hierarchyNode;
  if (pickedNode.isValid ())
    hierarchyNode = openCloseHierarchy->getHierarchyNode (pickedNode);
  GlABoundingCircle *boundingCircle =
    static_cast<GlABoundingCircle *> (pickedAug);

  if (!pickedNode.isValid() && (boundingCircle == NULL)) return;
  //if node valid, it should not be behind bounding circle, open node.
  else if (hierarchyNode.isValid()) {
    bool defaultAlgo = openCloseHierarchy->getUseDefaultAlgorithms ();
    if (layoutReset)
      openCloseHierarchy->setUseDefaultAlgorithms (false);
    grouseMorph->morphOpen (pickedNode, layoutReset);
    if (layoutReset)
      openCloseHierarchy->setUseDefaultAlgorithms (defaultAlgo);
    metaGraphHierarchy->synchListViewColour (hierarchyNode);
  }
  else if (boundingCircle != NULL) {
    if (boundingCircle->getSelected())
      metaGraphHierarchy->clearSelection();
    Coord boundingPos = boundingCircle->getPos ();
    node hierarchyNode = boundingCircle->getHierarchyNode ();
    grouseMorph->morphClosed (hierarchyNode, boundingPos, layoutReset);
  }
  else return;
}

//===============================================================
/**
 * Function to power open nodes in the hierarchy
 */

void MouseGrouseHierarchyNavigate::powerOpen (const QPoint &mouseClick,
				     GlGraphWidget *graphWidget) {
  node pickedNode;
  edge pickedEdge;
  GlAugmentedDisplay *pickedAug = NULL;
  pick (mouseClick.x(), mouseClick.y(), graphWidget, true, true, true, 
	false, false, pickedNode, pickedEdge, pickedAug);
  if (pickedNode.isValid()) 
    grouseMorph->powerOpenNodes (pickedNode);
}

//===============================================================
/**
 * Callback to handle mouse down events for grouse.  It also
 * listens to the keybord to see if any keys are down as well.
 * The graph widget is graph, and the Qt mouse event is e.
 */
bool MouseGrouseHierarchyNavigate::mPressEvent(GlGraphWidget *graph, QMouseEvent *e) {
  if ((e->button() == QT_OPEN_CLOSE_BUTTON) &&
      (e->state() == QT_OPEN_CLOSE_KEY_DOWN))
    openClose (e->pos(), graph, false);
  else if ((e->button() == QT_POWER_OPEN_BUTTON) &&
	   (e->state() == QT_POWER_OPEN_KEY_DOWN))
    powerOpen (e->pos(), graph);
  else if ((e->button() == QT_CLEAR_LAYOUT_BUTTON) &&
	   (e->state() == QT_CLEAR_LAYOUT_KEY_DOWN))
    openClose (e->pos(), graph, true);
  else {
    e->ignore();
    return false;
  }//end else
  return true;
}

//===============================================================
/**
 * Event filter for all the mouse events on the grouse view window.
 * Determines which private navigation function to call.
 */
bool MouseGrouseHierarchyNavigate::eventFilter (QObject *widget, QEvent *e) {
  GlGraphWidget *glGraphWidget = static_cast<GlGraphWidget *> (widget);
  if (e->type() == QEvent::MouseButtonPress) 
    return mPressEvent(glGraphWidget, static_cast<QMouseEvent *> (e));
  else return false;
  return true;
}
