#include <qcursor.h>
#include <tulip/BooleanProperty.h>
#include "MouseGrousePullOut.h"
#include "MouseGrouseGeneralOperations.h"

using namespace tlp;
using namespace std;

//===============================================================
//Defines for buttons to be used on each mouse operation
#define QT_PULL_OUT_BUTTON (Qt::RightButton)
#define QT_PULL_OUT_KEY_DOWN (Qt::ControlButton)

//===============================================================
inline void startPullOutTime (clock_t &start, time_t &startLongTime) {
  time (&startLongTime);
  start = clock ();
}//end startPullOutTime

//===============================================================
inline void endPullOutTime (clock_t &start, time_t &startLongTime) {
  clock_t end = clock();
  time_t endLongTime;
  time (&endLongTime);
  double diff, diffLong;
  diff = ((double) (end - start) / (double) CLOCKS_PER_SEC);
  diffLong = difftime (endLongTime, startLongTime);
  cout << "-------------------------------------------------------" << endl;
  cout << "Pull out took:" << endl;
  cout << "In short time " << diff << endl;
  cout << "In long time " <<  diffLong << endl; 
  cout << "-------------------------------------------------------" << endl;
}//end endPullOutTime

//===============================================================
/*
 * Default constructors and destructors
 */
MouseGrousePullOut::MouseGrousePullOut (GrouseMorphing *grouseMorph) :
  grouseMorph (grouseMorph) {}
MouseGrousePullOut::~MouseGrousePullOut() {}

//===============================================================
void MouseGrousePullOut::pullInOrPushOut (const QPoint &mouseClick, 
					  GlGraphWidget *graphWidget) {
  node pickedNode;
  edge pickedEdge;
  GlAugmentedDisplay *pickedAug = NULL;
  pick (mouseClick.x(), mouseClick.y(), graphWidget, true, true, 
	true, false, false, pickedNode, pickedEdge, pickedAug);
  if (pickedNode.isValid ()) {
      clock_t start;
      time_t startLongTime;
      startPullOutTime (start, startLongTime);
      grouseMorph->morphPullOut (pickedNode, 1);
      endPullOutTime (start, startLongTime);
  }//end if
}//end pullInOrPushOut

//===============================================================
/**
 * Callback to handle mouse down events for grouse.  It also
 * listens to the keybord to see if any keys are down as well.
 * The graph widget is graph, and the Qt mouse event is e.
 */
bool MouseGrousePullOut::mPressEvent(GlGraphWidget *graph, QMouseEvent *e) {
  if ((e->button() == QT_PULL_OUT_BUTTON) &&
      (e->state() == QT_PULL_OUT_KEY_DOWN)) {
    pullInOrPushOut (e->pos(), graph);
  }
  else {
    e->ignore();
    return false;
  }//end else
  return true;
}

//===============================================================
/**
 * Event filter for all the mouse events on the grouse view window.
 * Determines which private navigation function to call.
 */
bool MouseGrousePullOut::eventFilter (QObject *widget, QEvent *e) {
  GlGraphWidget *glGraphWidget = static_cast<GlGraphWidget *> (widget);
  if (e->type() == QEvent::MouseButtonPress) 
    return mPressEvent(glGraphWidget, static_cast<QMouseEvent *> (e));
  else return false;
  return true;
}


