#include <tulip/IntegerProperty.h>
#include <tulip/LayoutProperty.h>
#include <tulip/SizeProperty.h>
#include <tulip/DoubleProperty.h>
#include <tulip/StringProperty.h>
#include <tulip/ExtendedClusterOperation.h>
#include <tulip/ForEach.h>
#include <tulip/DrawingTools.h>
#include "OpenClose.h"
#include "ColourShapeDefaults.h"

using namespace std;
using namespace tlp;

//================================================================
node OpenClose::close (Graph *displayGraph, BooleanProperty &nodeSelection,
		       const Coord &pos,
		       const int shape, const Color &color, 
		       const double borderWidth, const Size &size,
		       const string &label) {
  set<node> tmp;
  node curNode;
  forEach (curNode, nodeSelection.getNodesEqualTo (true)) 
    tmp.insert (curNode);
  node groupedMetaNode = tlp::createMetaNode (displayGraph, tmp,
					      closedSubgraphs,
					      groupedMetaGraphs);
  IntegerProperty *shapes = 
    displayGraph->getProperty<IntegerProperty>("viewShape");
  shapes->setNodeValue (groupedMetaNode, shape);
  ColorProperty *colours = 
    displayGraph->getProperty<ColorProperty>("viewColor");
  colours->setNodeValue (groupedMetaNode, color);
  DoubleProperty *borders =
    displayGraph->getProperty<DoubleProperty> ("viewBorderWidth");
  borders->setNodeValue (groupedMetaNode, borderWidth);
  SizeProperty *sizes = displayGraph->getProperty<SizeProperty> ("viewSize");
  sizes->setNodeValue (groupedMetaNode, size);
  StringProperty *labels =
    displayGraph->getProperty<StringProperty> ("viewLabel");
  labels->setNodeValue (groupedMetaNode, label);
  closedSubgraphs->addNode (groupedMetaNode);
  edge addedEdge;
  forEach (addedEdge, closedSubgraphs->getInOutEdges (groupedMetaNode)) {
    closedSubgraphs->addEdge (addedEdge);
    colours->setEdgeValue (addedEdge, EDGE_COLOUR);
  }//end forEach
  return groupedMetaNode;
}//end close

//================================================================
void OpenClose::open (Graph *displayGraph, node metaNode) {
  ColorProperty *colours = 
    displayGraph->getProperty<ColorProperty>("viewColor");
  Graph *openSubGraph = getGroupedGraph (metaNode);
  vector<node> subgraphNodes;
  node curNode;
  forEach (curNode, openSubGraph->getNodes())
    subgraphNodes.push_back (curNode);
 
  SizeProperty *viewSize =
    displayGraph->getProperty<SizeProperty> ("viewSize");
  Graph *metaGraph = groupedMetaGraphs->getNodeValue (metaNode);
  pair<Coord, Coord> boundingBox = 
    tlp::computeBoundingBox (metaGraph, 
			     metaGraph
			     ->getProperty<LayoutProperty> ("viewLayout"),
			     metaGraph
			     ->getProperty<SizeProperty> ("viewSize"),
			     metaGraph
			     ->getProperty<DoubleProperty> ("viewRotation"));
  Coord c_size = boundingBox.first - boundingBox.second;
  Size size (c_size.getX(), c_size.getY(), c_size.getZ());
  viewSize->setNodeValue (metaNode, size);
  tlp::openMetaNode (displayGraph, metaNode, closedSubgraphs,
		     groupedMetaGraphs);
  for (unsigned int i = 0; i < subgraphNodes.size(); ++i) {
    edge curEdge;
    forEach (curEdge, displayGraph->getInOutEdges (subgraphNodes[i])) {
      colours->setEdgeValue (curEdge, EDGE_COLOUR);
      node opposite = displayGraph->opposite (curEdge, subgraphNodes[i]);
      if (!closedSubgraphs->existEdge (subgraphNodes[i], opposite).isValid())
	closedSubgraphs->addEdge (curEdge);
    }//end forEach
  }//end for
}//end open
