#include "HSVConversion.h"
#include "RotatingColours.h" 

using namespace std;
using namespace tlp;

#define NUM_COLOURS (5)
Color matchColourList [] = {GROUSE_PURPLE,
			    GROUSE_ORANGE_2,
			    GROUSE_BLUE,
			    GROUSE_GREEN,
			    GROUSE_TEAL};
#define NUM_SATS (4)			     
float saturations [] = {0.15, 0.40, 0.55, 0.75};

#define MATCH_SATURATION (0.90)
#define UNMATCH_SATURATION (0.45)


//================================================================
RotatingColours::RotatingColours () : 
  decompositionNumber (0), 
  greyScale (false) {}

//================================================================
void RotatingColours::setGreyScale (bool on) {
  greyScale = on;
}//end setGreyScale

//================================================================
bool RotatingColours::getGreyScale () {
  return greyScale;
}//end getGreyScale

//================================================================
Color RotatingColours::getCurMatchColour () {
  HSV curHSV;
  if (!greyScale)
    curHSV = rgb_to_hsv (matchColourList [decompositionNumber%NUM_COLOURS]);
  else
    curHSV = rgb_to_hsv (ROOT_COLOUR);
  curHSV.s = MATCH_SATURATION;
  return hsv_to_rgb (curHSV);
}//end getCurMatchColour

//================================================================
Color RotatingColours::getCurUnmatchColour () {
  HSV curHSV;
  if (!greyScale)
    curHSV = rgb_to_hsv (matchColourList [decompositionNumber%NUM_COLOURS]);
  else
    curHSV = rgb_to_hsv (ROOT_COLOUR);
  curHSV.s = UNMATCH_SATURATION;
  return hsv_to_rgb (curHSV);
}//end getCurMatchColour

//================================================================
Color RotatingColours::getNextCatColour () {
  //currently I don't know how to handle categories with colours
  if (!greyScale) return getCurMatchColour();
  else {
    float percentage = saturations [decompositionNumber%NUM_SATS];
    char grey = static_cast <char> (floor (255*percentage));
    ++decompositionNumber;
    return Color (grey, grey, grey, NODE_ALPHA);
  }//end else
}//end getNextCatColour

//================================================================
Color RotatingColours::getCategoryColour (string &curCategory) {
  stdext::hash_map<string, Color>::iterator catIt = 
    categoryHash.find (curCategory);
  if (catIt != categoryHash.end ())
    return catIt->second;
  else {
    Color nextColor = getNextCatColour ();
    categoryHash [curCategory] = nextColor;
    return nextColor;
  }//end else
}//end getCategoryColour

//================================================================
void RotatingColours::incrementColour () {
  ++decompositionNumber;
}

//================================================================
void RotatingColours::reset () {
  decompositionNumber = 0;
  categoryHash.clear ();
}
