/**
 * This class creates a rotating colour object.  The object keeps a
 * counter.  Whenever the counter is incremented, a new colour is
 * selected.  It also modifies the colour to be come more saturated
 * (matches) and less saturated (unmatches) when modifying the hierarchy.
 */
#ifndef __ROTATING_COLOURS_H__
#define __ROTATING_COLOURS_H__
#include <string>
#if (__GNUC__ < 3)
#  include <hash_map>
#else
#  include <ext/hash_map>
#endif
#include <tulip/hash_string.h>
#include "ColourShapeDefaults.h"

class RotatingColours {

 public:

  /**
   * Default constructor.  Initializes decomposition number to zero.
   */
  RotatingColours ();

  /**
   * Method to get the current match colour.
   */
  tlp::Color getCurMatchColour ();

  /**
   * Method to get the current unmatch colour.
   */
  tlp::Color getCurUnmatchColour ();

  /**
   * Method to increment to the next colour.
   */
  void incrementColour ();

  /**
   * Method to turn Grouseflocks into greyscale
   */
  void setGreyScale (bool on);

  /**
   * Returns true if we are in greyscale and false otherwise.
   */
  bool getGreyScale ();

  /**
   * Method to get the current category color given the category
   * value
   */
  tlp::Color getCategoryColour (std::string &curCategory);

  /**
   * Resets all the colours for category and matches
   */
  void reset ();

 private:
  unsigned int decompositionNumber; //the number of decompositions done
  bool greyScale; //if true, the colours are greyscale
  stdext::hash_map<std::string, tlp::Color> categoryHash;

  /**
   * Computes the next category colour.
   */
  tlp::Color getNextCatColour ();
  
};

#endif
