#include <string>
#include <qinputdialog.h>
#include <tulip/ForEach.h>
#include <tulip/StableIterator.h>
#include "AttributeEqualValue.h"

using namespace std;
using namespace tlp;

#define NO_MATCH_EQUAL_VALUE (string ("()"))

namespace {
  const char * paramHelp[] = {
    // selectedNodes
    HTML_HELP_OPEN() \
    HTML_HELP_DEF( "type", "PropertyInterface*" ) \
    HTML_HELP_BODY() \
    "Specify the property that will be used to partition the graph" \
    HTML_HELP_CLOSE(),
  };
}
//================================================================
AttributeEqualValue::AttributeEqualValue (const PropertyContext &context):
  tlp::DoubleAlgorithm(context)  {
  addParameter<PropertyInterface*>("Property", paramHelp[0]);
}

AttributeEqualValue::~AttributeEqualValue(){}

//================================================================
inline PropertyInterface *promptEquProperty (Graph *graph) {
  QStringList propertyChoices;
  Iterator <string> *properties = graph->getProperties();
  while (properties->hasNext()) {
    string curProp = properties->next();
    if (curProp.find ("view") == string::npos)
      propertyChoices << curProp;
  } delete properties; //end while
  bool ok = false;
  string propertyName = QInputDialog::getItem ("Grouse",
					       "Select an equal value property:",
					       propertyChoices, 0, 
					       FALSE, &ok);
  if (ok) return graph->getProperty (propertyName);
  else return NULL;
}

//================================================================
bool AttributeEqualValue::run () {
  bool hasMatches = false;
  BooleanProperty *matches = 0;
  DataSet localData;
  PropertyInterface *metricClusters = 0;
  if (dataSet != 0) {
    dataSet->get("Property", metricClusters);
    hasMatches = dataSet->get ("matches", matches);
  }//end if
  if (metricClusters == 0) {
    metricClusters = promptEquProperty (graph);
    if (metricClusters == NULL) return false;
  }//end if

  //cluster them
  localData.set("Property", metricClusters);
  string errMsg;
  bool success = tlp::applyAlgorithm (graph, errMsg, &localData, 
				      "Equal Value");
  if (!success) {
    cerr << "Equal Value clustering returned the following error:\n"
	 << errMsg << "\n(Is the plugin properly installed?)" << endl;
    return false;
  }//end if

  double maxMetric = 0;
  double localMax = 0;
  StableIterator<Graph *> equalValClusters (graph->getSubGraphs());
  while (equalValClusters.hasNext()) {
    localMax = 0;
    Graph *curCluster = equalValClusters.next();
    bool matched = 
      metricClusters->getNodeStringValue (curCluster->getOneNode ()) !=
      NO_MATCH_EQUAL_VALUE;
    DoubleProperty connectedComponents (curCluster);
    success = curCluster->computeProperty ("Connected Component",
					   &connectedComponents,
					   errMsg);
    if (!success) {
      cerr << "Connected components returned the following error " << endl 
	   << errMsg << endl << "(Is the plugin installed?)" << endl;
      return false;
    }//end if

    node curNode;
    forEach (curNode, curCluster->getNodes()) {
      double nodeValue = connectedComponents.getNodeValue (curNode); 
      DoubleAlgorithm::doubleResult->setNodeValue (curNode, 
						   nodeValue + maxMetric);
      if (nodeValue > localMax) localMax = nodeValue;
      if (hasMatches) matches->setNodeValue (curNode, matched);
    }//end forEach
    maxMetric += localMax + 1;
    graph->delAllSubGraphs (curCluster);
  } //end while
  return true;
}//end run

DOUBLEPLUGIN(AttributeEqualValue,"Attribute Equal Value Clustering","Daniel Archambault","02/12/2007","Alpha","0","1");
